
import logging
import json
from datetime import datetime
from solaredge_interface.utils.timedates import datetime_to_string, FORMAT_DATETIME_TIMEZONE_STRING


logger = logging.getLogger(__name__)
DATA_PARSE_ERROR_LOGGER = False


class JSONEncoderDateTime(json.JSONEncoder):

    def default(self, obj):
        if isinstance(obj, datetime):
            return datetime_to_string(obj, datetime_format=FORMAT_DATETIME_TIMEZONE_STRING)
        else:
            return json.JSONEncoder.default(self, obj)


def json_decode(string, error_logger=DATA_PARSE_ERROR_LOGGER):
    try:
        data = json.loads(string)
    except json.decoder.JSONDecodeError:
        if error_logger:
            logger.error('Unable to JSON decode: {}'.format(string[0:255]))
        data = None
    return data




import os
import shelve
import logging
import tempfile
import functools

from solaredge_interface import __title__ as NAME
from solaredge_interface import __solaredge_api_baseurl__ as BASEURL
from solaredge_interface.exceptions.SolarEdgeInterfaceException import SolarEdgeInterfaceException
from solaredge_interface.utils.url_join import url_join, url_join_site_ids
from solaredge_interface.utils.http_request import http_request
from solaredge_interface.utils.json import json_decode

logger = logging.getLogger(__name__)


class SolarEdgeAPI:
    """
    This class implements Python3 interfaces to the documented SolarEdge API end-points.  Refer to
    [se_monitoring_api.pdf](https://www.solaredge.com/sites/default/files/se_monitoring_api.pdf) for more details
    on the SolarEdge API.
    """

    api_key = None
    datetime_response = None
    pandas_response = None

    def __init__(self, api_key, datetime_response=False, pandas_response=False):
        """
        To call the SolarEdge API you need a valid `api_key` which can be obtained from your SolarEdge account.

        _parameters_
        * _api_key_ (str) required - a valid api_key from https://monitoring.solaredge.com
        * _datetime_response_ (bool) default: False - if True then parse all fields with a date or datetime string and
        convert them into timezone aware Python datetime objects.
        * _pandas_response_ (bool) default: False - if True then parse response data and flatten into Pandas DataFrame
        and make available in the `.pandas` response attribute
        """
        if not api_key:
            raise SolarEdgeInterfaceException('Must provide a SolarEdge api_key value.')
        self.api_key = api_key
        self.datetime_response = datetime_response
        self.pandas_response = pandas_response

    @functools.lru_cache()
    def get_accounts(self, size=100, start_index=0, search_text="", sort_property="", sort_order="ASC"):
        """
        Returns a list of sub-accounts (if available) that are accessible by the `api_key` with an ability to
        search and filter the results.

        NB: `api_key` that do not have access to sub-accounts will return a "not authorized" response.

        _parameters_
        * _size_ (int) default: `100` - The maximum number of accounts returned by this call.  The maximum number of
        accounts that can be returned by this call is 100. If you have more than 100 accounts, just request another
        100 accounts with startIndex=100 which will fetch accounts 100-199.
        * _start_index_ (int) default: `0` - The first account index to be returned in the results.
        * _search_text_ (str) default: - The search text for accounts.  Searchable accounts properties: Name, Notes,
        Email, Country, State, City, Zip, Full address
        * _sort_property_ (str) default: - A sorting option for this account list, based on one of its properties.
        Available sort properties: Name, country, city, Address, zip, fax, phone, notes
        * _sort_order_ (str) default: `ASC` - Sort order for the sort property. Allowed values are ASC (ascending) and
        DESC (descending)

        Uses Least-Recently-Used caching strategy to reduce calls to API backend and speed re-occurring function calls.
        """
        url = url_join(BASEURL, "accounts", "list")
        params = {
            'api_key': self.api_key,
            'size': size,
            'startIndex': start_index,
            'sortOrder': sort_order,
        }
        if search_text:
            params['searchText'] = search_text
        if sort_property:
            params['sortProperty'] = sort_property
        return self.__response_wrapper(http_request(url, params))

    @functools.lru_cache()
    def get_sites(self, size=100, start_index=0, search_text="", sort_property="", sort_order="ASC", status="Active,Pending"):
        """
        Returns the sites accessible by the `api_key` with an ability to search and filter.

        _parameters_
        * _size_ (int) default: `100` - The maximum number of sites returned by this call.  The maximum number of sites
        that can be returned by this call is 100. If you have more than 100 sites, just request another 100 sites with
        startIndex=100 which will fetch sites 100-199.
        * _start_index_ (int) default: `0` - The first site index to be returned in the results.
        * _search_text_ (str) default: - The search text for sites.  Searchable site properties: Name, Notes,
        Address, City, Zip code, Full address, Country
        * _sort_property_ (str) default: - A sorting option for this site list, based on one of its properties.
        Available sort properties: Name, Country, State, City, Address, Zip, Status, PeakPower, InstallationDate,
         Amount, MaxSeverity, CreationTime
        * _sort_order_ (str) default: `ASC` - Sort order for the sort property. Allowed values are ASC (ascending) and
        DESC (descending)
        * _status_ (str) default: `Active,Pending` - Select the sites to be included in the list by their status:
        Active, Pending, Disabled, All.

        Uses Least-Recently-Used caching strategy to reduce calls to API backend and speed re-occurring function calls.
        """
        url = url_join(BASEURL, "sites", "list")
        params = {
            'api_key': self.api_key,
            'size': size,
            'startIndex': start_index,
            'sortOrder': sort_order,
            'status': status
        }
        if search_text:
            params['searchText'] = search_text
        if sort_property:
            params['sortProperty'] = sort_property
        return self.__response_wrapper(http_request(url, params))

    @functools.lru_cache()
    def get_site_details(self, site_id):
        """
        Returns site details for `site_id` such as name, location, status, etc.

        _parameters_
        * _site_id_ (int) required - The site identifier to retrieve data for.

        Uses Least-Recently-Used caching strategy to reduce calls to API backend and speed re-occurring function calls.
        """
        url = url_join(BASEURL, "site", site_id, "details")
        params = {
            'api_key': self.api_key
        }
        return self.__response_wrapper(http_request(url, params), site_id=site_id)

    @functools.lru_cache()
    def get_site_timezone(self, site_id, tempfile_cache_use=True):
        """
        Returns site timezone for `site_id` - returns from local tempfile cache to prevent repeated requests.  This
        function is provided as a convenience.

        _parameters_
        * _site_id_ (int) required - The site identifier to retrieve data for.

        Uses Least-Recently-Used caching strategy to reduce calls to API backend and speed re-occurring function calls.
        """
        if ',' in str(site_id):
            return None
        if tempfile_cache_use:
            temp_filename = os.path.join(tempfile.gettempdir(), '{}.cache'.format(NAME))
            with shelve.open(temp_filename) as cached:
                key = '{}.timezone'.format(str(site_id).strip())
                logger.debug('get_site_timezone; cache key={}'.format(key))
                if key not in cached:
                    logger.debug('get_site_timezone; value not cached, adding to cache file {}'.format(temp_filename))
                    response = http_request(url_join(BASEURL, "site", site_id, "details"), {'api_key': self.api_key})
                    response.data = json_decode(response.text)
                    cached[key] = response.data['details']['location']['timeZone']
                else:
                    logger.debug('get_site_timezone; value from cache file {}'.format(temp_filename))
                tz = cached[key]
        else:
            logger.debug('get_site_timezone; no cache use')
            response = http_request(url_join(BASEURL, "site", site_id, "details"), {'api_key': self.api_key})
            response.data = json_decode(response.text)
            tz = response.data['details']['location']['timeZone']
        return tz

    @functools.lru_cache()
    def get_site_data_period(self, site_id):
        """
        Returns the start-date and end-date of energy production at the site(s).

        _parameters_
        * _site_id_ (int or list) required - The site identifier(s) to retrieve data for, may be provided as a single
        int value or a list of int values to retrieve data in "bulk-mode"

        Uses Least-Recently-Used caching strategy to reduce calls to API backend and speed re-occurring function calls.
        """
        url = url_join(BASEURL, url_join_site_ids(site_id), 'dataPeriod')
        params = {
            'api_key': self.api_key
        }
        return self.__response_wrapper(http_request(url, params), site_id=site_id)

    def get_site_energy(self, site_id, start_date, end_date, time_unit="DAY"):
        """
        Returns the site(s) energy measurements.

        NB: the time input parameters required are date values in the format YYYY-MM-DD not full timestamp values.

        _parameters_
        * _site_id_ (int or list) required - The site identifier(s) to retrieve data for, may be provided as a single
        int value or a list of int values to retrieve data in "bulk-mode"
        * _start_date_ (str) required - must be in format YYYY-MM-DD
        * _end_date_ (str) required - must be in format YYYY-MM-DD
        * _time_unit_ (str) default: `DAY` - Permitted values are: QUARTER_OF_AN_HOUR, HOUR, DAY, WEEK, MONTH, YEAR
        """
        url = url_join(BASEURL, url_join_site_ids(site_id), 'energy')
        params = {
            'api_key': self.api_key,
            'startDate': start_date,
            'endDate': end_date,
            'timeUnit': time_unit
        }
        return self.__response_wrapper(http_request(url, params), site_id=site_id)

    def get_site_time_frame_energy(self, site_id, start_date, end_date):
        """
        Return the site(s) total energy produced for a given date period.

        NB: the time input parameters required are date values in the format YYYY-MM-DD not full timestamp values.

        _parameters_
        * _site_id_ (int or list) required - The site identifier(s) to retrieve data for, may be provided as a single
        int value or a list of int values to retrieve data in "bulk-mode"
        * _start_date_ (str) required - must be in format YYYY-MM-DD
        * _end_date_ (str) required - must be in format YYYY-MM-DD
        """
        url = url_join(BASEURL, url_join_site_ids(site_id), 'timeFrameEnergy')
        params = {
            'api_key': self.api_key,
            'startDate': start_date,
            'endDate': end_date
        }
        return self.__response_wrapper(http_request(url, params), site_id=site_id)

    def get_site_overview(self, site_id):
        """
        Return the site(s) overview data.

        _parameters_
        * _site_id_ (int or list) required - The site identifier(s) to retrieve data for, may be provided as a single
        int value or a list of int values to retrieve data in "bulk-mode"
        """
        url = url_join(BASEURL, url_join_site_ids(site_id), 'overview')
        params = {
            'api_key': self.api_key
        }
        return self.__response_wrapper(http_request(url, params), site_id=site_id)

    def get_site_power(self, site_id, start_time, end_time):
        """
        Return the site(s) power measurements in 15 minute resolution.

        _parameters_
        * _site_id_ (int or list) required - The site identifier(s) to retrieve data for, may be provided as a single
        int value or a list of int values to retrieve data in "bulk-mode"
        * _start_time_ (str) required - must be in format YYYY-MM-DD hh:mm:ss
        * _end_time_ (str) required - must be in format YYYY-MM-DD hh:mm:ss
        """
        url = url_join(BASEURL, url_join_site_ids(site_id), 'power')
        params = {
            'api_key': self.api_key,
            'startTime': start_time,
            'endTime': end_time
        }
        return self.__response_wrapper(http_request(url, params), site_id=site_id)

    def get_site_power_details(self, site_id, start_time, end_time, meters=None):
        """
        Detailed site power measurements from meters such as consumption, export (feed-in), import (purchase), etc.
        Calculated meter readings (also referred to as "virtual meters"), such as self-consumption, are calculated
        using the data measured by the meter and the inverters.

        _parameters_
        * _site_id_ (int) required - The site identifier to retrieve data for.
        * _start_time_ (str) required - must be in format YYYY-MM-DD hh:mm:ss
        * _end_time_ (str) required - must be in format YYYY-MM-DD hh:mm:ss
        * _meters_ (str) default: - If this value is omitted all meter readings are returned. The following values are
        permitted separated by comma: Production, Consumption, SelfConsumption, FeedIn, Purchased
        """
        url = url_join(BASEURL, "site", site_id, "powerDetails")
        params = {
            'api_key': self.api_key,
            'startTime': start_time,
            'endTime': end_time
        }
        if meters:
            params['meters'] = meters
        return self.__response_wrapper(http_request(url, params), site_id=site_id)

    def get_site_energy_details(self, site_id, start_time, end_time, meters=None, time_unit="DAY"):
        """
        Detailed site energy measurements from meters such as consumption, export (feed-in), import (purchase), etc.
        Calculated meter readings (also referred to as "virtual meters"), such as self-consumption, are calculated
        using the data measured by the meter and the inverters.

        _parameters_
        * _site_id_ (int) required - The site identifier to retrieve data for.
        * _start_time_ (str) required - must be in format YYYY-MM-DD hh:mm:ss
        * _end_time_ (str) required - must be in format YYYY-MM-DD hh:mm:ss
        * _meters_ (str) default: None - If this value is omitted all meter readings are returned. The following values
        are permitted separated by comma: Production, Consumption, SelfConsumption, FeedIn, Purchased
        * _time_unit_ (str) default: `DAY` - Permitted values are: QUARTER_OF_AN_HOUR, HOUR, DAY, WEEK, MONTH, YEAR
        """
        url = url_join(BASEURL, "site", site_id, "energyDetails")
        params = {
            'api_key': self.api_key,
            'startTime': start_time,
            'endTime': end_time,
            'timeUnit': time_unit
        }
        if meters:
            params['meters'] = meters
        return self.__response_wrapper(http_request(url, params), site_id=site_id)

    def get_site_current_power_flow(self, site_id):
        """
        Provides the current power flow between all elements of the site including PV array, storage (battery), loads
        (consumption) and grid.

        _parameters_
        * _site_id_ (int) required - The site identifier to retrieve data for.
        """
        url = url_join(BASEURL, "site", site_id, "currentPowerFlow")
        params = {
            'api_key': self.api_key
        }
        return self.__response_wrapper(http_request(url, params), site_id=site_id)

    def get_site_storage_data(self, site_id, start_time, end_time, serials=None):
        """
        Get detailed storage information from batteries:the state of energy, power and lifetime energy.

        _parameters_
        * _site_id_ (int) required - The site identifier to retrieve data for.
        * _start_time_ (str) required - must be in format YYYY-MM-DD hh:mm:ss
        * _end_time_ (str) required - must be in format YYYY-MM-DD hh:mm:ss
        * _serials_ (list) default: None - Return data only for specific battery serial numbers; If omitted, the
        response includes all the batteries at the site.
        """
        url = url_join(BASEURL, "site", site_id, "storageData")
        params = {
            'api_key': self.api_key,
            'startTime': start_time,
            'endTime': end_time
        }
        if serials:
            params['serials'] = serials
        return self.__response_wrapper(http_request(url, params), site_id=site_id)

    # def get_site_image(self, site_id, name=None, max_width=None, max_height=None, hash=None):
    #     pass

    # def get_site_installer_logo(self, site_id, name=None):
    #     pass

    def get_site_environmental_benefits(self, site_id, system_units=None):
        """
        Get all environmental benefits based on site energy production:CO2 emissions saved, equivalent trees planted,
        and light bulbs powered for a day.

        _parameters_
        * _site_id_ (int) required - The site identifier to retrieve data for.
        * _system_units_ (str) default: None - The system units used when returning gas emission savings.  Valid
        values: `Metrics`, `Imperial` note these values are case sensitive. If system_units is not specified, the user
        system units are used.
        """
        url = url_join(BASEURL, "site", site_id, "envBenefits")
        params = {
            'api_key': self.api_key,
        }
        if system_units:
            params['systemUnits'] = system_units
        return self.__response_wrapper(http_request(url, params), site_id=site_id)

    # def get_site_equipment_list(self, site_id):
    #     pass

    @functools.lru_cache()
    def get_site_inventory(self, site_id):
        """
        Get the inventory of SolarEdge equipment at the site, including inverters/SMIs, batteries, meters,  gateways
        and sensors.

        _parameters_
        * _site_id_ (int) required - The site identifier to retrieve data for.

        Uses Least-Recently-Used caching strategy to reduce calls to API backend and speed re-occurring function calls.
        """
        url = url_join(BASEURL, "site", site_id, "inventory")
        params = {
            'api_key': self.api_key
        }
        return self.__response_wrapper(http_request(url, params), site_id=site_id)

    def get_site_equipment_data(self, site_id, start_time, end_time, serial_number):
        """
        Get specific inverter data for a given timeframe.

        _parameters_
        * _site_id_ (int) required - The site identifier to retrieve data for.
        * _start_time_ (str) required - must be in format YYYY-MM-DD hh:mm:ss
        * _end_time_ (str) required - must be in format YYYY-MM-DD hh:mm:ss
        * _serial_number_ (str) required - The inverter short serial number, eg 12345678-90
        """
        url = url_join(BASEURL, "equipment", site_id, serial_number, "data")
        params = {
            'api_key': self.api_key,
            'startTime': start_time,
            'endTime': end_time
        }
        return self.__response_wrapper(http_request(url, params), site_id=site_id)

    def get_site_equipment_change_log(self, site_id, serial_number):
        """
        Returns a list of equipment component replacements ordered by date. This method is applicable to inverters,
        optimizers, batteries and gateways.

        _parameters_
        * _site_id_ (int) required - The site identifier to retrieve data for.
        * _serial_number_ (str) required - Inverter, battery, optimizer or gateway short serial number.
        """
        url = url_join(BASEURL, "equipment", site_id, serial_number, "changeLog")
        params = {
            'api_key': self.api_key,
        }
        return self.__response_wrapper(http_request(url, params), site_id=site_id)

    def get_site_meters(self, site_id, start_time, end_time, meters=None):
        """
        Returns for each meter on site its lifetime energy reading, metadata and the device to which it is connected.

        _parameters_
        * _site_id_ (int) required - The site identifier to retrieve data for.
        * _start_time_ (str) required - must be in format YYYY-MM-DD hh:mm:ss
        * _end_time_ (str) required - must be in format YYYY-MM-DD hh:mm:ss
        * _meters_ (str ot list) default: None - Select specific meters only. If this value is omitted, all meter
        readings are returned.  Valid values: Production, Consumption,
         FeedIn, Purchased.
        """
        url = url_join(BASEURL, "site", site_id, "meters")
        params = {
            'api_key': self.api_key,
            'startTime': start_time,
            'endTime': end_time
        }
        if meters:
            params['meters'] = meters
        return self.__response_wrapper(http_request(url, params), site_id=site_id)

    def get_site_equipment_sensors(self, site_id):
        """
        Returns a list of all the sensors in the site, and the device to which they are connected.

        _parameters_
        * _site_id_ (int) required - The site identifier to retrieve data for.
        """
        url = url_join(BASEURL, "equipment", site_id, "sensors")
        params = {
            'api_key': self.api_key,
        }
        return self.__response_wrapper(http_request(url, params), site_id=site_id)

    def get_version_current(self):
        """
        Return the most updated version number in <major.minor.revision> format.
        """
        url = url_join(BASEURL, "version", "current")
        params = {
            'api_key': self.api_key,
        }
        return self.__response_wrapper(http_request(url, params))

    def get_version_supported(self):
        """
        Return a list of supported version numbers in <major.minor.revision> format
        """
        url = url_join(BASEURL, "version", "supported")
        params = {
            'api_key': self.api_key,
        }
        return self.__response_wrapper(http_request(url, params))

    def __response_wrapper(self, response, site_id=None, parse_response=True, pandas_column_trim=None):
        if parse_response:
            response.data = json_decode(response.text)
            if response.data:
                if self.datetime_response:
                    try:
                        data_to_datetime
                    except NameError:
                        logger.debug('from solaredge_interface.utils.timedates import data_to_datetime')
                        from solaredge_interface.utils.timedates import data_to_datetime
                    response.data = data_to_datetime(data=response.data)

                if site_id:
                    try:
                        set_datetime_tzinfo
                    except NameError:
                        logger.debug('from solaredge_interface.utils.timedates import set_datetime_tzinfo')
                        from solaredge_interface.utils.timedates import set_datetime_tzinfo
                    response.data = set_datetime_tzinfo(data=response.data, tz=self.get_site_timezone(site_id))

                if self.pandas_response:
                    try:
                        data_to_pandas
                    except NameError:
                        logger.debug('solaredge_interface.utils.pandas import data_to_pandas')
                        from solaredge_interface.utils.pandas import data_to_pandas
                    response.pandas = data_to_pandas(data=response.data, prefix_to_remove=pandas_column_trim)
        return response




import logging
from solaredge_interface.exceptions.SolarEdgeInterfaceException import SolarEdgeInterfaceException
from solaredge_interface import __env_site_id__ as ENV_SITE_ID
from solaredge_interface import __output_format_default__ as OUTPUT_FORMAT_DEFAULT
from solaredge_interface.utils.timedates import datestring_current, datestring_days_delta
from solaredge_interface.utils.timedates import timestring_current, timestring_seconds_delta


logger = logging.getLogger(__name__)


def site_id(kwargs, config):
    if kwargs['site_id'] is None:
        kwargs['site_id'] = config.site_id
    if kwargs['site_id'] is None:
        raise SolarEdgeInterfaceException('Must provide a "site_id" input value.  See documentation to set via command '
                                          'line argument or using the {} environment variable.'.format(ENV_SITE_ID))
    logger.debug('site_id: {}'.format(kwargs['site_id']))
    return kwargs


def end_date(kwargs):
    if 'end_date' not in kwargs.keys() or kwargs['end_date'] is None:
        kwargs['end_date'] = datestring_current()
    logger.debug('end_date: {}'.format(kwargs['end_date']))
    return kwargs


def start_date(kwargs, delta_days=-7):
    if 'start_date' not in kwargs.keys() or kwargs['start_date'] is None:
        kwargs['start_date'] = datestring_days_delta(kwargs['end_date'], delta=delta_days)
    logger.debug('start_date: {}'.format(kwargs['start_date']))
    return kwargs


def end_time(kwargs):
    if 'end_time' not in kwargs.keys() or kwargs['end_time'] is None:
        kwargs['end_time'] = timestring_current()
    logger.debug('end_time: {}'.format(kwargs['end_time']))
    return kwargs


def start_time(kwargs, delta_time=-(3600*24*7)):
    if 'start_time' not in kwargs.keys() or kwargs['start_time'] is None:
        kwargs['start_time'] = timestring_seconds_delta(kwargs['end_time'], delta=delta_time)
    logger.debug('start_time: {}'.format(kwargs['start_time']))
    return kwargs




import os
import logging
import functools
import configparser

from solaredge_interface import __env_api_key__ as ENV_API_KEY
from solaredge_interface import __env_site_id__ as ENV_SITE_ID
from solaredge_interface import __env_output_format__ as ENV_OUTPUT_FORMAT
from solaredge_interface import __config_file_user__ as CONFIG_FILE_USER
from solaredge_interface import __config_file_system__ as CONFIG_FILE_SYSTEM
from solaredge_interface import __config_section_name__ as CONFIG_SECTION_NAME
from solaredge_interface.exceptions.SolarEdgeInterfaceException import SolarEdgeInterfaceException


logger = logging.getLogger(__name__)


class ConfigException(SolarEdgeInterfaceException):
    pass


class Config:

    session_config_file = None

    def __init__(self, session_config_file=None):
        self.session_config_file = session_config_file

    def __getattr__(self, item):
        return self.get(item)

    @functools.lru_cache()
    def get(self, item):

        if not self.session_config_file:

            if item == 'api_key':
                env_name = ENV_API_KEY
            elif item == 'site_id':
                env_name = ENV_SITE_ID
            elif item == 'format':
                env_name = ENV_OUTPUT_FORMAT
            else:
                raise ConfigException('Unknown configuration attribute requested.', item)

            # Environment variable
            if os.environ.get(env_name):
                logger.debug('"{}" returned from environment variable: {}'.format(item, env_name))
                return os.environ.get(env_name)

        # Configuration file based config setting
        return self.__get_config_from_file(item)

    @functools.lru_cache()
    def __get_config_from_file(self, item):

        config = None

        if self.session_config_file and not os.path.isfile(os.path.expanduser(self.session_config_file)):
            raise ConfigException('Unable to find the configuration filename supplied.', self.session_config_file)

        config_files = [
            self.session_config_file,
            CONFIG_FILE_USER,
            CONFIG_FILE_SYSTEM
        ]

        for config_file in config_files:
            if config_file and os.path.isfile(os.path.expanduser(config_file)):
                config = self.__load_config_file(config_file)
                break

        if type(config) is not dict:
            logger.debug('"{}" unset because no configuration file found.'.format(item))
            return None

        _item = item.replace('_', '').replace('-', '')
        if _item not in config.keys():
            logger.debug('Unable to find "{}" setting in the configuration file.'.format(item))
            return None

        logger.debug('"{}" returned from the configuration file.'.format(item))
        return config[_item]

    @functools.lru_cache()
    def __load_config_file(self, filename, section=CONFIG_SECTION_NAME):

        filename = os.path.expanduser(filename)
        if os.path.isfile(filename):

            cp = configparser.ConfigParser()

            try:
                cp.read(filename)
            except Exception as e:
                raise ConfigException('Unable to correctly parse the configuration file provided.', e)

            if section not in cp.sections():
                raise ConfigException(
                    'Unable to locate the "{}" section in configuration file.'.format(section), filename
                )
            config = {}
            for option in cp.options(section):
                config[str(option).replace('_', '').replace('-', '')] = cp.get(section, option)
            logger.debug('configuration file read: {}'.format(filename))
            return config

        raise ConfigException('Unable to find the configuration filename supplied.', filename)




from pytz import timezone
from dateutil import parser
from datetime import datetime, timedelta

FORMAT_DATE_STRING = '%Y-%m-%d'
FORMAT_DATETIME_STRING = '%Y-%m-%d %H:%M:%S'
FORMAT_DATETIME_TIMEZONE_STRING = '%Y-%m-%d %H:%M:%S %Z%z'
DICT_KEY_CONTAIN_CONVERT_DATETIME = ['date', 'time']  # keys containing these stings will attempt string to datetime


def datestring_current(tz=None, datetime_format=FORMAT_DATE_STRING):
    if tz:
        dt = datetime.now(tz=timezone(tz))
    else:
        dt = datetime.now()
    return dt.strftime(datetime_format)


def datestring_days_delta(datestring, delta, datetime_format=FORMAT_DATE_STRING):
    dt = parser.parse(datestring)
    dt_delta = dt + timedelta(days=delta)
    return dt_delta.strftime(datetime_format)


def timestring_current(tz=None, datetime_format=FORMAT_DATETIME_STRING):
    if tz:
        dt = datetime.now(tz=timezone(tz))
    else:
        dt = datetime.now()
    return dt.strftime(datetime_format)


def timestring_seconds_delta(timestring, delta, datetime_format=FORMAT_DATETIME_STRING):
    dt = parser.parse(timestring)
    dt_delta = dt + timedelta(seconds=delta)
    return dt_delta.strftime(datetime_format)


def datetime_to_string(dt, datetime_format=FORMAT_DATETIME_STRING):
    value = dt.strftime(datetime_format)
    return value


def string_to_datetime(string, tz=None):
    try:
        dt = parser.parse(string)
    except (ValueError, TypeError):
        return string
    if tz and not dt.tzinfo:
        dt = timezone(tz).localize(dt)
    return dt


def data_to_datetime(data, tz=None):
    if type(data) is list:
        for index, item in enumerate(data):
            data[index] = data_to_datetime(item)
    elif type(data) is dict:
        for key in data.keys():
            if type(data[key]) is str:
                for key_pattern in DICT_KEY_CONTAIN_CONVERT_DATETIME:
                    if key_pattern in key.lower():
                        data[key] = string_to_datetime(data[key], tz)
                        break
            else:
                data[key] = data_to_datetime(data[key])
    return data


def set_datetime_tzinfo(data, tz=None):
    if type(data) is list:
        for index, item in enumerate(data):
            data[index] = set_datetime_tzinfo(item, tz)
    elif type(data) is dict:
        for key in data.keys():
            data[key] = set_datetime_tzinfo(data[key], tz)
    elif type(data) is datetime and data.tzinfo is None and tz:
        data = timezone(tz).localize(data)
    return data




import re
import collections
import pandas as pd


def data_to_pandas(data, sep='.', prefix_to_remove=None):
    column_names, data_table = tabelize_data(
        data=flatten_data(data, sep=sep),
        sep=sep,
        prefix_remove=prefix_to_remove
    )
    return pd.DataFrame.from_dict(data_table, orient='index', columns=column_names)


def flatten_data(data, sep='.'):
    obj = collections.OrderedDict()

    def data_recurse(t, parent_key=''):
        if isinstance(t, list):
            for index in range(len(t)):
                data_recurse(t[index], parent_key + sep + str(index) if parent_key else str(index))
        elif isinstance(t, dict):
            for key, value in t.items():
                data_recurse(value, parent_key + sep + key if parent_key else key)
        else:
            obj[parent_key] = t

    data_recurse(data)
    return obj


def tabelize_data(data, sep='.', prefix_remove=None):

    def column_name_row_group(key_name):
        pattern_string = '\\{}'.format(sep) + '[0-9]+' + '\\{}'.format(sep)
        pattern = re.compile(pattern_string)
        matches = pattern.findall(key_name)
        column_index_key = ''
        if matches:
            for match in matches:
                key_name = key_name.replace(match, sep)
                column_index_key = '{}{}'.format(column_index_key, match.replace(sep,''))
        if prefix_remove and key_name.startswith(prefix_remove):
            return key_name[len(prefix_remove):], column_index_key
        return key_name, column_index_key

    # scan for column_names and row_groups
    column_names = []
    column_index_depth = 0
    for key, _ in data.items():
        column_name, row_group = column_name_row_group(key)
        if len(row_group) > column_index_depth:
            column_index_depth = len(row_group)
        if column_name not in column_names:
            column_names.append(column_name)

    # group data together in row_groups with column_names; maintain a row_group counter index
    data_group = collections.OrderedDict()
    row_group_index = 0
    row_group_index_map = collections.OrderedDict()
    for key, value in data.items():
        column_name, row_group_short = column_name_row_group(key)
        row_group = row_group_short.ljust(column_index_depth, '0')
        if row_group not in data_group.keys():
            data_group[row_group] = collections.OrderedDict()
            row_group_index_map[row_group] = row_group_index
            row_group_index += 1
        data_group[row_group][column_name] = value

    # map data into an ordered per row dict
    data_table = collections.OrderedDict()
    previous_group_index_key = '0' * column_index_depth
    for group_index_key, group_index_value in row_group_index_map.items():
        row_name = 'row_{}'.format(group_index_value)
        data_table[row_name] = []
        for column_name in column_names:
            if column_name not in data_group[group_index_key].keys():
                if column_name in data_group[previous_group_index_key].keys():
                    data_group[group_index_key][column_name] = data_group[previous_group_index_key][column_name]
                else:
                    data_group[group_index_key][column_name] = None
            data_table[row_name].append(data_group[group_index_key][column_name])
        previous_group_index_key = group_index_key

    return column_names, data_table




import logging
import requests
from solaredge_interface import __http_request_user_agent__ as USER_AGENT
from solaredge_interface import __http_request_timeout__ as REQUESTS_TIMEOUT


logger = logging.getLogger(__name__)


class Response(object):
    url = request = headers = cookies = status_code = elapsed = text = None

    def __init__(self, **attrs):
        for k in attrs:
            setattr(self, k, attrs[k])

    def __str__(self):
        if not hasattr(self, 'text'):
            return None
        return str(self.text)

    def __set__(self, name, value):
        setattr(self, name, value)


def http_request(url, params=None, headers=None, timeout=REQUESTS_TIMEOUT):

    if type(params) is dict:
        for key in params:
            if type(params[key]) is list:
                params[key] = ','.join(str(params[key]).strip(' '))
            params[key] = str(params[key]).strip(' ')

    if type(headers) is dict:
        headers['user-agent'] = USER_AGENT
    else:
        headers = {'user-agent': USER_AGENT}

    r = requests.get(url, params=params, headers=headers, timeout=timeout)

    response = Response(
        url=r.url,
        request=r.request,
        headers=r.headers,
        cookies=r.cookies,
        status_code=r.status_code,
        text=r.text,
        elapsed=r.elapsed,
    )
    logger.debug('http-response; url={}'.format(r.url))
    logger.debug('http-response; http-status={}'.format(r.status_code))
    return response



# -*- coding: utf8 -*-
# Copyright (c) 2020 Nicholas de Jong

__title__ = "solaredge-interface"
__author__ = "Nicholas de Jong <contact@nicholasdejong.com>"
__version__ = '0.3.2'
__license__ = "MIT"

__env_api_key__ = 'SOLAREDGE_API_KEY'
__env_site_id__ = 'SOLAREDGE_SITE_ID'
__env_output_format__ = 'SOLAREDGE_OUTPUT_FORMAT'

__output_format_default__ = 'json'

__config_file_user__ = '~/.solaredge-interface'
__config_file_system__ = '/etc/solaredge-interface'
__config_section_name__ = 'solaredge-interface'

__solaredge_api_baseurl__ = 'https://monitoringapi.solaredge.com'
__http_request_timeout__ = 10
__http_request_user_agent__ = '{}/{}'.format(__title__, __version__)





def url_join(*parts):
    """
    Join terms together with forward slashes
    """

    # first strip extra forward slashes (except http:// and the likes) and
    # create list
    part_list = []
    for part in parts:
        p = str(part)
        if p.endswith('//'):
            p = p[0:-1]
        else:
            p = p.strip('/')
        part_list.append(p)

    # join everything together
    url = '/'.join(part_list)
    return url


def url_join_site_ids(data):

    if type(data) is list:
        return url_join('sites', ','.join(data).replace(' ',''))
    elif ',' in str(data):
        return url_join('sites', data.replace(' ',''))
    else:
        return url_join('site', data.replace(' ',''))




from solaredge_interface.cli import click
from solaredge_interface import __title__ as NAME
from solaredge_interface import __version__ as VERSION
from solaredge_interface.exceptions.SolarEdgeInterfaceException import SolarEdgeInterfaceException


def solaredge_interface():

    try:
        click.solaredge_interface()
    except SolarEdgeInterfaceException as e:
        print('')
        print('{} v{}'.format(NAME, VERSION))
        print('ERROR: ', end='')
        for err in iter(e.args):
            print(err)
        print('')
        exit(9)




import json
import logging
from solaredge_interface.utils.json import JSONEncoderDateTime
from solaredge_interface import __output_format_default__ as OUTPUT_FORMAT_DEFAULT
from solaredge_interface.exceptions.SolarEdgeInterfaceException import SolarEdgeInterfaceException

logger = logging.getLogger(__name__)


def format_output(response, output_format=OUTPUT_FORMAT_DEFAULT):
    if type(output_format) is str:
        output_format = output_format.lower()
    if hasattr(response, 'data') and response.data and output_format == 'json':
        output_json(response.data)
    elif hasattr(response, 'pandas') and response.pandas is not None and output_format == 'pandas':
        output_pandas(response.pandas)
    elif hasattr(response, 'pandas') and response.pandas is not None and output_format == 'csv':
        output_csv(response.pandas)
    elif output_format not in ['json', 'pandas', 'csv']:
        raise SolarEdgeInterfaceException('Unknown output format requested: {}'.format(output_format))
    else:
        logging.warning('response.data is not available for output formatting, raw response data is provided')
        print(response.text)


def output_json(data):
    print(json.dumps(data, cls=JSONEncoderDateTime, indent='  '))


def output_pandas(pandas_dataframe):
    output_json(json.loads(pandas_dataframe.to_json()))


def output_csv(pandas_dataframe):
    print(pandas_dataframe.to_csv())




import sys
import click
import logging
import warnings

from solaredge_interface import __version__ as VERSION
from solaredge_interface import __env_api_key__ as ENV_API_KEY
from solaredge_interface import __output_format_default__ as OUTPUT_FORMAT_DEFAULT
from solaredge_interface.utils import arg_helper
from solaredge_interface.utils.output import format_output
from solaredge_interface.api.SolarEdgeAPI import SolarEdgeAPI
from solaredge_interface.cli.config import Config
from solaredge_interface.exceptions.SolarEdgeInterfaceException import SolarEdgeInterfaceException

solaredge_api = None
solaredge_cli_config = None


@click.group()
@click.option('-c', '--config', help='Override default config ~/.solaredge-interface')
@click.option('-f', '--format', help='Output format; csv, json, pandas (default: json)')
@click.option('-v', '--verbose', is_flag=True, help='Verbose logging messages (debug level).')
@click.option('-q', '--quiet', is_flag=True, help='Quiet mode, with priority over --verbose')
@click.option('-W', '--disable-warnings', is_flag=True, help='Disable Python warnings.')
@click.version_option(VERSION)
def solaredge_interface(config, format, verbose, quiet, disable_warnings):
    """
    The solaredge-interface provides a command-line interface to interact with the Python SolarEdgeAPI module which
    itself calls the SolarEdge public API endpoints at https://monitoringapi.solaredge.com making it even easier to
    access your data from SolarEdge.

    Configuration can be achieved through command arguments, environment values or config file.

    Documentation available https://solaredge-interface.readthedocs.io
    """

    ctx = click.get_current_context()
    ctx.ensure_object(dict)

    if quiet:
        level = logging.CRITICAL
    elif verbose:
        level = logging.DEBUG
    else:
        level = logging.WARNING

    logging.basicConfig(
        level=level,
        format='[%(levelname)s|%(asctime)s|%(name)s]: %(message)s',
    )

    if not sys.warnoptions and not disable_warnings:
        warnings.simplefilter("default")  # Change the filter in this process

    global solaredge_api
    global solaredge_cli_config

    solaredge_cli_config = Config(session_config_file=config)
    if not solaredge_cli_config.api_key:
        raise SolarEdgeInterfaceException('SolarEdge api_key value not supplied.  See documentation to set this '
                                          'using the {} environment variable or using configuration file(s).'
                                          .format(ENV_API_KEY))
    if format:
        solaredge_cli_config.format = format
    elif solaredge_cli_config.format is None:
        solaredge_cli_config.format = OUTPUT_FORMAT_DEFAULT

    solaredge_api = SolarEdgeAPI(api_key=solaredge_cli_config.api_key, datetime_response=True, pandas_response=True)


@solaredge_interface.command('accounts')
@click.option('--size', help='The maximum number of accounts returned by this call', required=False, default=100)
@click.option('--start_index', help='The first account index to be returned in the results', required=False, default=0)
@click.option('--search_text', help='The search text for accounts', required=False, default="")
@click.option('--sort_property', help='A sorting option based on one of its properties', required=False, default="")
@click.option('--sort_order', help='Sort order for the sort property', required=False, default="ASC")
def get_accounts(**kwargs):
    """
    Get the accessible >sub< accounts.

    NB: the response from this endpoint returns a "Not authorized" message if there are no sub-accounts assigned.
    """
    format_output(
        response=solaredge_api.get_accounts(**kwargs),
        output_format=solaredge_cli_config.format
    )


@solaredge_interface.command('sites')
@click.option('--size', help='The maximum number of sites returned by this call', required=False, default=100)
@click.option('--start_index', help='The first site index to be returned in the results', required=False, default=0)
@click.option('--search_text', help='The search text for sites', required=False, default="")
@click.option('--sort_property', help='A sorting option based on one of its properties', required=False, default="")
@click.option('--sort_order', help='Sort order for the sort property', required=False, default="ASC")
@click.option('--status', help='Selected sites to be included in the list', required=False, default="Active,Pending")
def get_sites(**kwargs):
    """
    Get the list of accessible sites
    """
    format_output(
        response=solaredge_api.get_sites(**kwargs),
        output_format=solaredge_cli_config.format
    )


@solaredge_interface.command('site_details')
@click.argument('site_id', required=False)
def get_site_details(**kwargs):
    """
    Get site details; name, location, status, etc.
    """
    kwargs = arg_helper.site_id(kwargs, config=solaredge_cli_config)
    format_output(
        response=solaredge_api.get_site_details(**kwargs),
        output_format=solaredge_cli_config.format
    )


@solaredge_interface.command('site_data_period')
@click.argument('site_id', required=False)
def get_site_data_period(**kwargs):
    """
    Sites(s) start_date and end_date of production
    """
    kwargs = arg_helper.site_id(kwargs, config=solaredge_cli_config)
    format_output(
        response=solaredge_api.get_site_data_period(**kwargs),
        output_format=solaredge_cli_config.format
    )


@solaredge_interface.command('site_energy')
@click.argument('site_id', required=False)
@click.option('--start_date', help='Default 7 days ago, else format "YYYY-MM-DD"')
@click.option('--end_date', help='Default todays date, else format "YYYY-MM-DD"')
@click.option('--time_unit', help='QUARTER_OF_AN_HOUR, HOUR, *DAY, WEEK, MONTH, YEAR', default='DAY')
def get_site_energy(**kwargs):
    """
    Site(s) energy measurements
    """
    kwargs = arg_helper.site_id(kwargs, config=solaredge_cli_config)
    kwargs = arg_helper.end_date(kwargs)
    kwargs = arg_helper.start_date(kwargs, delta_days=-7)
    format_output(
        response=solaredge_api.get_site_energy(**kwargs),
        output_format=solaredge_cli_config.format
    )


@solaredge_interface.command('site_time_frame_energy')
@click.argument('site_id', required=False)
@click.option('--start_date', help='Default 7 days ago, else format "YYYY-MM-DD"')
@click.option('--end_date', help='Default todays date, else format "YYYY-MM-DD"')
def get_site_time_frame_energy(**kwargs):
    """
    Site(s) total energy produced for a given period
    """
    kwargs = arg_helper.site_id(kwargs, config=solaredge_cli_config)
    kwargs = arg_helper.end_date(kwargs)
    kwargs = arg_helper.start_date(kwargs, delta_days=-7)
    format_output(
        response=solaredge_api.get_site_time_frame_energy(**kwargs),
        output_format=solaredge_cli_config.format
    )


@solaredge_interface.command('site_overview')
@click.argument('site_id', required=False)
def get_site_overview(**kwargs):
    """
    Sites(s) overview data
    """
    kwargs = arg_helper.site_id(kwargs, config=solaredge_cli_config)
    format_output(
        response=solaredge_api.get_site_overview(**kwargs),
        output_format=solaredge_cli_config.format
    )


@solaredge_interface.command('site_power')
@click.argument('site_id', required=False)
@click.option('--start_time', help='Default 7 days ago, else format "YYYY-MM-DD hh:mm:ss"')
@click.option('--end_time', help='Default now time, else format "YYYY-MM-DD hh:mm:ss"')
def get_site_power(**kwargs):
    """
    Site(s) power measurements
    """
    kwargs = arg_helper.site_id(kwargs, config=solaredge_cli_config)
    kwargs = arg_helper.end_time(kwargs)
    kwargs = arg_helper.start_time(kwargs, delta_time=-(3600*24*7))
    format_output(
        response=solaredge_api.get_site_power(**kwargs),
        output_format=solaredge_cli_config.format
    )


@solaredge_interface.command('site_power_details')
@click.argument('site_id', required=False)
@click.option('--start_time', help='Default 7 days ago, else format "YYYY-MM-DD hh:mm:ss"')
@click.option('--end_time', help='Default now time, else format "YYYY-MM-DD hh:mm:ss"')
@click.option('--meters', help='Production, Consumption, SelfConsumption, FeedIn, Purchased', default=None)
def get_site_power_details(**kwargs):
    """
    Detailed site power measurements from meters
    """
    kwargs = arg_helper.site_id(kwargs, config=solaredge_cli_config)
    kwargs = arg_helper.end_time(kwargs)
    kwargs = arg_helper.start_time(kwargs, delta_time=-(3600*24*7))
    format_output(
        response=solaredge_api.get_site_power_details(**kwargs),
        output_format=solaredge_cli_config.format
    )


@solaredge_interface.command('site_energy_details')
@click.argument('site_id', required=False)
@click.option('--start_time', help='Default 7 days ago, else format "YYYY-MM-DD hh:mm:ss"')
@click.option('--end_time', help='Default now time, else format "YYYY-MM-DD hh:mm:ss"')
@click.option('--meters', help='Production, Consumption, SelfConsumption, FeedIn, Purchased', default=None)
@click.option('--time_unit', help='QUARTER_OF_AN_HOUR, HOUR, DAY, WEEK, MONTH, YEAR', default='DAY')
def get_site_energy_details(**kwargs):
    """
    Detailed site energy measurements from meters
    """
    kwargs = arg_helper.site_id(kwargs, config=solaredge_cli_config)
    kwargs = arg_helper.end_time(kwargs)
    kwargs = arg_helper.start_time(kwargs, delta_time=-(3600*24*7))
    format_output(
        response=solaredge_api.get_site_energy_details(**kwargs),
        output_format=solaredge_cli_config.format
    )


@solaredge_interface.command('site_current_power_flow')
@click.argument('site_id', required=False)
def get_site_current_power_flow(**kwargs):
    """
    Current power flow between all elements of the site
    """
    kwargs = arg_helper.site_id(kwargs, config=solaredge_cli_config)
    format_output(
        response=solaredge_api.get_site_current_power_flow(**kwargs),
        output_format=solaredge_cli_config.format
    )


@solaredge_interface.command('site_storage_data')
@click.argument('site_id', required=False)
@click.option('--start_time', help='Default 7 days ago, else format "YYYY-MM-DD hh:mm:ss"')
@click.option('--end_time', help='Default now time, else format "YYYY-MM-DD hh:mm:ss"')
@click.option('--serials', help='If omitted, returns all batteries at site', default=None)
def get_site_storage_data(**kwargs):
    """
    Detailed storage information from batteries
    """
    kwargs = arg_helper.site_id(kwargs, config=solaredge_cli_config)
    kwargs = arg_helper.end_time(kwargs)
    kwargs = arg_helper.start_time(kwargs, delta_time=-(3600*24*7))
    format_output(
        response=solaredge_api.get_site_storage_data(**kwargs),
        output_format=solaredge_cli_config.format
    )


@solaredge_interface.command('site_environmental_benefits')
@click.argument('site_id', required=False)
@click.option('--system_units', help='Metrics, Imperial - case sensitive', default=None)
def get_site_environmental_benefits(**kwargs):
    """
    Environmental benefits based on site energy production
    """
    kwargs = arg_helper.site_id(kwargs, config=solaredge_cli_config)
    format_output(
        response=solaredge_api.get_site_environmental_benefits(**kwargs),
        output_format=solaredge_cli_config.format
    )


@solaredge_interface.command('site_inventory')
@click.argument('site_id', required=False)
def get_site_inventory(**kwargs):
    """
    Inventory of SolarEdge equipment at the site
    """
    kwargs = arg_helper.site_id(kwargs, config=solaredge_cli_config)
    format_output(
        response=solaredge_api.get_site_inventory(**kwargs),
        output_format=solaredge_cli_config.format
    )


@solaredge_interface.command('site_equipment_data')
@click.argument('site_id', required=False)
@click.option('--start_time', help='Default 7 days ago, else format "YYYY-MM-DD hh:mm:ss"')
@click.option('--end_time', help='Default now time, else format "YYYY-MM-DD hh:mm:ss"')
@click.option('--serial_number', help='The inverter short serial number', required=True)
def get_site_equipment_data(**kwargs):
    """
    Get specific inverter data for a given timeframe
    """
    kwargs = arg_helper.site_id(kwargs, config=solaredge_cli_config)
    kwargs = arg_helper.end_time(kwargs)
    kwargs = arg_helper.start_time(kwargs, delta_time=-(3600*24*7))
    format_output(
        response=solaredge_api.get_site_equipment_data(**kwargs),
        output_format=solaredge_cli_config.format
    )


@solaredge_interface.command('site_equipment_change_log')
@click.argument('site_id', required=False)
@click.option('--serial_number', help='The inverter short serial number', required=True)
def get_site_equipment_change_log(**kwargs):
    """
    Equipment component replacements ordered by date
    """
    kwargs = arg_helper.site_id(kwargs, config=solaredge_cli_config)
    format_output(
        response=solaredge_api.get_site_equipment_change_log(**kwargs),
        output_format=solaredge_cli_config.format
    )


@solaredge_interface.command('site_meters')
@click.argument('site_id', required=False)
@click.option('--start_time', help='Default 7 days ago, else format "YYYY-MM-DD hh:mm:ss"')
@click.option('--end_time', help='Default now time, else format "YYYY-MM-DD hh:mm:ss"')
@click.option('--meters', help='Production, Consumption, SelfConsumption, FeedIn, Purchased', default=None)
def get_site_meters(**kwargs):
    """
    Meter lifetime energy, metadata and connection detail
    """
    kwargs = arg_helper.site_id(kwargs, config=solaredge_cli_config)
    kwargs = arg_helper.end_time(kwargs)
    kwargs = arg_helper.start_time(kwargs, delta_time=-(3600*24*7))
    format_output(
        response=solaredge_api.get_site_meters(**kwargs),
        output_format=solaredge_cli_config.format
    )


@solaredge_interface.command('site_equipment_sensors')
@click.argument('site_id', required=False)
def get_site_equipment_sensors(**kwargs):
    """
    Sensors in the site and connections
    """
    kwargs = arg_helper.site_id(kwargs, config=solaredge_cli_config)
    format_output(
        response=solaredge_api.get_site_equipment_sensors(**kwargs),
        output_format=solaredge_cli_config.format
    )


@solaredge_interface.command('version_current')
def get_version_current(**kwargs):
    """
    Current version in <major.minor.revision> format
    """
    format_output(
        response=solaredge_api.get_version_current(**kwargs),
        output_format=solaredge_cli_config.format
    )


@solaredge_interface.command('version_supported')
def get_version_supported(**kwargs):
    """
    Supported version numbers in <major.minor.revision> format.
    """
    format_output(
        response=solaredge_api.get_version_supported(**kwargs),
        output_format=solaredge_cli_config.format
    )
